@@ -209,7 +209,7 @@
    code that does the save/restore is generated by the linker, so
    we have no good way to determine at compile time what to do.  */
 
-#ifdef __powerpc64__
+#ifdef __64BIT__
 #define MD_FROB_UPDATE_CONTEXT(CTX, FS)					\
   do {									\
     if ((FS)->regs.reg[2].how == REG_UNSAVED)				\
